@@ -1,78 +1,67 @@
 /*
-    Copyright (C) 2001, S.R.Haque <srhaque@iee.org>.
-    Copyright (C) 2002, David Faure <david@mandrakesoft.com>
     This file is part of the KDE project
+    SPDX-FileCopyrightText: 2001 S.R. Haque <srhaque@iee.org>.
+    SPDX-FileCopyrightText: 2002 David Faure <david@mandrakesoft.com>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2, as published by the Free Software Foundation.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LGPL-2.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "kreplacedialog.h"
 #include "kfinddialog_p.h"
 
 #include <QCheckBox>
-#include <QRegExp>
 #include <QLineEdit>
 #include <QGridLayout>
 #include <QGroupBox>
+#include <QRegularExpression>
 
-#include <khistorycombobox.h>
-#include <klocalizedstring.h>
-#include <kmessagebox.h>
+#include <KHistoryComboBox>
+#include <KLocalizedString>
+#include <KMessageBox>
 
 /**
  * we need to insert the strings after the dialog is set
  * up, otherwise QComboBox will deliver an aweful big sizeHint
  * for long replacement texts.
  */
-class KReplaceDialogPrivate
+class KReplaceDialogPrivate : public KFindDialogPrivate
 {
+    Q_DECLARE_PUBLIC(KReplaceDialog)
+
 public:
     explicit KReplaceDialogPrivate(KReplaceDialog *q)
-        : q(q)
+        : KFindDialogPrivate(q)
     {
     }
 
     void _k_slotOk();
 
     QStringList replaceStrings;
-    KReplaceDialog *q = nullptr;
-    QWidget *replaceExtension = nullptr;
+    mutable QWidget *replaceExtension = nullptr;
     bool initialShowDone = false;
 };
 
 KReplaceDialog::KReplaceDialog(QWidget *parent, long options, const QStringList &findStrings,
                                const QStringList &replaceStrings, bool hasSelection)
-    : KFindDialog(parent, options, findStrings, hasSelection, true /*create replace dialog*/),
-      d(new KReplaceDialogPrivate(this))
+    : KFindDialog(*new KReplaceDialogPrivate(this), parent, options, findStrings, hasSelection, true /*create replace dialog*/)
 {
+    Q_D(KReplaceDialog);
+
     d->replaceStrings = replaceStrings;
 }
 
-KReplaceDialog::~KReplaceDialog()
-{
-    delete d;
-}
+KReplaceDialog::~KReplaceDialog() = default;
 
 void KReplaceDialog::showEvent(QShowEvent *e)
 {
+    Q_D(KReplaceDialog);
+
     if (!d->initialShowDone) {
         d->initialShowDone = true; // only once
 
         if (!d->replaceStrings.isEmpty()) {
             setReplacementHistory(d->replaceStrings);
-            KFindDialog::d->replace->lineEdit()->setText(d->replaceStrings[0]);
+            d->replace->lineEdit()->setText(d->replaceStrings[0]);
         }
     }
 
@@ -81,13 +70,15 @@ void KReplaceDialog::showEvent(QShowEvent *e)
 
 long KReplaceDialog::options() const
 {
+    Q_D(const KReplaceDialog);
+
     long options = 0;
 
     options = KFindDialog::options();
-    if (KFindDialog::d->promptOnReplace->isChecked()) {
+    if (d->promptOnReplace->isChecked()) {
         options |= PromptOnReplace;
     }
-    if (KFindDialog::d->backRef->isChecked()) {
+    if (d->backRef->isChecked()) {
         options |= BackReference;
     }
     return options;
@@ -95,9 +86,11 @@ long KReplaceDialog::options() const
 
 QWidget *KReplaceDialog::replaceExtension() const
 {
+    Q_D(const KReplaceDialog);
+
     if (!d->replaceExtension) {
-        d->replaceExtension = new QWidget(KFindDialog::d->replaceGrp);
-        KFindDialog::d->replaceLayout->addWidget(d->replaceExtension, 3, 0, 1, 2);
+        d->replaceExtension = new QWidget(d->replaceGrp);
+        d->replaceLayout->addWidget(d->replaceExtension, 3, 0, 1, 2);
     }
 
     return d->replaceExtension;
@@ -105,14 +98,18 @@ QWidget *KReplaceDialog::replaceExtension() const
 
 QString KReplaceDialog::replacement() const
 {
-    return KFindDialog::d->replace->currentText();
+    Q_D(const KReplaceDialog);
+
+    return d->replace->currentText();
 }
 
 QStringList KReplaceDialog::replacementHistory() const
 {
-    QStringList lst = KFindDialog::d->replace->historyItems();
+    Q_D(const KReplaceDialog);
+
+    QStringList lst = d->replace->historyItems();
     // historyItems() doesn't tell us about the case of replacing with an empty string
-    if (KFindDialog::d->replace->lineEdit()->text().isEmpty()) {
+    if (d->replace->lineEdit()->text().isEmpty()) {
         lst.prepend(QString());
     }
     return lst;
@@ -120,31 +117,39 @@ QStringList KReplaceDialog::replacementHistory() const
 
 void KReplaceDialog::setOptions(long options)
 {
+    Q_D(KReplaceDialog);
+
     KFindDialog::setOptions(options);
-    KFindDialog::d->promptOnReplace->setChecked(options & PromptOnReplace);
-    KFindDialog::d->backRef->setChecked(options & BackReference);
+    d->promptOnReplace->setChecked(options & PromptOnReplace);
+    d->backRef->setChecked(options & BackReference);
 }
 
 void KReplaceDialog::setReplacementHistory(const QStringList &strings)
 {
+    Q_D(KReplaceDialog);
+
     if (!strings.isEmpty()) {
-        KFindDialog::d->replace->setHistoryItems(strings, true);
+        d->replace->setHistoryItems(strings, true);
     } else {
-        KFindDialog::d->replace->clearHistory();
+        d->replace->clearHistory();
     }
 }
 
 void KReplaceDialogPrivate::_k_slotOk()
 {
+    Q_Q(KReplaceDialog);
+
     // If regex and backrefs are enabled, do a sanity check.
-    if (q->KFindDialog::d->regExp->isChecked() && q->KFindDialog::d->backRef->isChecked()) {
-        QRegExp r(q->pattern());
-        int caps = r.captureCount();
-        QRegExp check(QStringLiteral("((?:\\\\)+)(\\d+)"));
-        int p = 0;
-        QString rep = q->replacement();
-        while ((p = check.indexIn(rep, p)) > -1) {
-            if (check.cap(1).length() % 2 && check.cap(2).toInt() > caps) {
+    if (regExp->isChecked() && backRef->isChecked()) {
+        const QRegularExpression re(q->pattern());
+        const int caps = re.captureCount();
+
+        const QString rep = q->replacement();
+        const QRegularExpression check(QStringLiteral("((?:\\\\)+)(\\d+)"));
+        auto iter = check.globalMatch(rep);
+        while (iter.hasNext()) {
+            const QRegularExpressionMatch match = iter.next();
+            if ((match.captured(1).size() % 2) && match.captured(2).toInt() > caps) {
                 KMessageBox::information(q, i18n(
                                              "Your replacement string is referencing a capture greater than '\\%1', ",  caps) +
                                          (caps ?
@@ -154,13 +159,11 @@ void KReplaceDialogPrivate::_k_slotOk()
                                          i18n("\nPlease correct."));
                 return; // abort OKing
             }
-            p += check.matchedLength();
         }
-
     }
 
-    q->KFindDialog::d->_k_slotOk();
-    q->KFindDialog::d->replace->addToHistory(q->replacement());
+    _k_slotOk();
+    replace->addToHistory(q->replacement());
 }
 
 #include "moc_kreplacedialog.cpp"
